#include <stdio.h>
#include <stdlib.h>

#define MEMSIZE (1<<14)

double memory[MEMSIZE];

const char* OPS[5] = { "LTZ", "LEZ", "EQZ", "GEZ", "GTZ" };


void emit_instruction(unsigned int code, unsigned int pos) {
    unsigned int op = code >> 28;
    if (op == 0) {
        // S-Type
        unsigned int imm = (code >> 21) & 0x7;
        unsigned int r1 = code & 0x3fff;
        op = code >> 24;
        switch(op) {
        case 0: // Noop
            printf("Noop()\n");
            break;
        case 1: // Cmpz
            if (imm > 4) {
                fprintf(stderr, "ERROR: unknown instruction %08x\n", code);
                exit(1);
            }
            printf("Cmpz(%u, %s)\n", r1, OPS[imm]);
            break;
        case 2: // Sqrt
            printf("Sqrt(%u, %u)\n", pos, r1);
            break;
        case 3: // Copy
            printf("Copy(%u, %u)\n", pos, r1);
            break;
        case 4: // Input
            printf("Input(%u, %u)\n", pos, r1);
            break;
        default:
            fprintf(stderr, "ERROR: unknown instruction %08x\n", code);
            exit(1);
        }
    }
    else {
        // D-Type
        unsigned int r1 = (code >> 14) & 0x3fff;
        unsigned int r2 = code & 0x3fff;
        switch(op) {
        case 1: // Add
            printf("Add(%u, %u, %u)\n", pos, r1, r2);
            break;
        case 2: // Sub
            printf("Sub(%u, %u, %u)\n", pos, r1, r2);
            break;
        case 3: // Mult
            printf("Mult(%u, %u, %u)\n", pos, r1, r2);
            break;
        case 4: // Div
            printf("Div(%u, %u, %u)\n", pos, r1, r2);
            break;
        case 5: // Output
            printf("Output(%u, %u)\n", r1, r2);
            break;
        case 6: // Phi
            printf("Phi(%u, %u, %u)\n", pos, r1, r2);
            break;
        default:
            fprintf(stderr, "ERROR: unknown instruction %08x\n", code);
            exit(1);
        }
    }
}

int main(void) {
    unsigned int pos = 0, end;

    printf("BEGIN_OVA\n");

    for(;;) {
        unsigned int code;

        fread(&memory[pos], sizeof(double), 1, stdin);
        fread(&code, sizeof(unsigned int), 1, stdin);
        if (feof(stdin))
            break;
        emit_instruction(code, pos);
        pos++;

        fread(&code, sizeof(unsigned int), 1, stdin);
        fread(&memory[pos], sizeof(double), 1, stdin);
        if (feof(stdin))
            break;
        emit_instruction(code, pos);
        pos++;
    }
    end = pos;

    printf("END_OVA\n");

    printf("BEGIN_MEMMAP\n");
    for(pos = 0; pos < end; pos++) {
        printf("%.16e,\n", memory[pos]);
    }
    printf("END_MEMMAP\n");

    return 0;
}
