#include <algorithm>
#include <complex>
#include <iomanip>
#include <iostream>
#include <cassert>
#include <cmath>
#include <cstring>
#include "../../include/ovm.h"

extern "C" const unsigned int TEAM_ID = 652;

using namespace std;

typedef complex<double> P;
const P I(0, 1);

const double G = 6.67428e-11;
const double Re = 6.357e6;
const double Me = 6.0e24;

P current_position() {
  return -P(read_sensor(2), read_sensor(3));
}

P current_target(int i) {
  return P(read_sensor(i * 3 + 7), read_sensor(i * 3 + 8)) + current_position();
}

P current_fuelstation() {
  return P(read_sensor(4), read_sensor(5)) + current_position();
}

P current_moon() {
  return P(read_sensor(0x64), read_sensor(0x65)) + current_position();
}

double current_fuel() {
    return read_sensor(1);
}

void set_thrust(const P& p) {
    set_thrust(p.real(), p.imag());
}

double inner_product(const P p1, const P p2) {
  return real(conj(p1) * p2);
}

double outer_product(const P p1, const P p2) {
  return imag(conj(p1) * p2);
}

const double PI=3.141592653589793238462;

double run_output() {
  double res = vm_run();
  P cur = current_position();

  if(get_timestep() % 50 == 0){
    cout << "VIS " << get_timestep()
         << " FATE " << real(cur) << " " << imag(cur) << " 0 0" << endl;
    for(int i = 0; i < 12; ++i){
      P target = current_target(i);
      cout << "VIS " << get_timestep()
           << " " << i << " " << real(target) << " " << imag(target) << " 0 0" << endl;
    }
    P fuel = current_fuelstation();
    cout << "VIS " << get_timestep()
         << " FUEL " << real(fuel) << " " << imag(fuel) << " 0 0" << endl;
    P moon = current_moon();
    cout << "VIS " << get_timestep()
         << " MOON " << real(moon) << " " << imag(moon) << " 0 0" << endl;
  }

  return res;
}

int main(int argc, char* argv[])
{
  cout.setf(ios::fixed);
  cout << setprecision(10);
  if (argc == 1) {
    printf("usage: clearsky-getlog {4001|4002|4003|4004}\n");
    return 0;
  }
  vm_init(atoi(argv[1]));
  for(int i = 0; i < 1000000; ++i){
    run_output();
  }
  return 0;
}
