#include <algorithm>
#include <complex>
#include <iomanip>
#include <iostream>
#include <cassert>
#include <cmath>
#include <cstring>
#include "ovm.hpp"

using namespace std;

#include "simulate.hpp"
#include "autofollow.hpp"


#define PERMISSIBLE_DISTANCE 1000

inline P next_v(const P& pos, const P& v) {
  P next_pos, next_v;
  simulate_v(pos, v, 1, next_pos, next_v);
  return next_v;
}

void auto_follow(OVM& ovm, int target_id, int initT, int minT) {
    if (minT < 0)
        minT = initT;

    printf("auto-follow: start following ");
    if (target_id == STATION_ID)
        printf("fuel station\n");
    else
        printf("target %d\n", target_id);

    ovm.print_status(target_id);

    int start_timestep = ovm.current_timestep();
    double start_fuel = ovm.current_fuel();

    // 速度を揃える + 10くらいのスピードで相手に近づく
    while(!ovm.finished() && !ovm.target_collected(target_id)) {
      if (abs(ovm.current_target_velocity(target_id) - ovm.current_fate_velocity()) <= 100 &&
	  abs(ovm.current_target_position(target_id) - ovm.current_fate_position()) <= 500) {
	printf("auto-follow: set velocity to traget.\n");
	ovm.print_status();

	while (abs(ovm.current_target_velocity(target_id) - ovm.current_fate_velocity()) > 0.1 && !ovm.finished()) {
	  ovm.run(next_v(ovm.current_target_position(target_id),
			 ovm.current_target_velocity(target_id))
		  - next_v(ovm.current_fate_position(),
			   ovm.current_fate_velocity()));
	}

	printf("auto-follow: start randebu-.\n");
	ovm.print_status();
	for (int i = 0; i < 1000; ++i) {
	  ovm.run();
	  ovm.print_vis();
	}
	if (ovm.target_collected(target_id) || ovm.finished()) {
	  break;
	}
	printf("auto-follow: Failed to follow ... :-(\n");
	ovm.print_status();
      }
      
      P diff_pos = ovm.current_target_position(target_id) -
	ovm.current_fate_position();
      P next_vel = next_v(ovm.current_target_position(target_id),
			 ovm.current_target_velocity(target_id))
		  - next_v(ovm.current_fate_position(),
			   ovm.current_fate_velocity());
      ovm.run(next_vel + diff_pos * 0.1);

      ovm.print_vis();
    }

    int end_timestep = ovm.current_timestep();
    double end_fuel = ovm.current_fuel();

    printf("auto-follow: finished. took %d seconds, %.0f fuel.\n", end_timestep-start_timestep, start_fuel-end_fuel);
    ovm.print_status();
}

