#include <algorithm>
#include <complex>
#include <iomanip>
#include <iostream>
#include <cassert>
#include <cmath>
#include <cstring>
#include "ovm.hpp"

using namespace std;

#include "guess_ellipse.hpp"


#define PREVIOUS_POSITION() (target_id == FATE_ID ? ovm.previous_fate_position() : ovm.previous_target_position(target_id))
#define CURRENT_POSITION()  (target_id == FATE_ID ? ovm.current_fate_position() : ovm.current_target_position(target_id))

/*
static double interpolate(const P& before, const P& middle, const P& after) {
    return (abs(middle-before) / (abs(after-middle) + abs(before-middle)));
}

ellipse_t guess_ellipse_by_earth(OVM& ovm_original, int target_id) {
    elipse_t e;

    OVM ovm(ovm_original);

    e.dir = (arg(CURRENT_POSITION() / PREVIOUS_POSITION()) > 0 ? 1 : -1);

    P start = CURRENT_POSITION();
    int start_timestep = ovm.current_timestep();
    e.center = start;
    e.period = 1;
    int n = 1;

    ovm.run();

    int last_sign = 1;
    for(;;) {

        int sign = (arg(CURRENT_POSITION() / start) * e.dir > 0 ? 1 : -1);
        if (last_sign == -1 && sign == 1) {
            // 一周した
            cerr << "DEBUG: interpolate = " << PREVIOUS_POSITION() << "--" << start << CURRENT_POSITION() << ", " << interpolate(PREVIOUS_POSITION(), start, CURRENT_POSITION()) << endl;
            e.period += interpolate(PREVIOUS_POSITION(), start, CURRENT_POSITION());
            break;
        }
        e.period += 1;
        e.center += CURRENT_POSITION();
        n++;

        last_sign = sign;
        ovm.run();

    }

    // 重心をとる
    e.center /= n;
    e.another_focus = e.center * 2.0;

    double d = abs(start) + abs(start-e.another_focus);
    e.major_radius = d / 2;
    e.minor_radius = sqrt(d*d/4 - norm(e.center));

    double perigee_arg = -arg(e.center);
    if (!(-5 <= perigee_arg && perigee_arg <= 5)) {
        // ほぼ真円。とりあえず、今の位置からちょっと先がapogeeということにしておく
        perigee_arg = arg(start * polar(1.0, 0.1*e.dir));
    }

    e.perigee = e.center + polar(e.major_radius, perigee_arg);
    e.apogee = e.center - polar(e.major_radius, perigee_arg);

    {
        P start_normalized = (start - e.center) * polar(1.0, -perigee_arg);
        start_normalized = P(start_normalized.real()/e.major_radius,
                             start_normalized.imag()/e.minor_radius);
        double start_to_perigee_dirarg = -arg(start_normalized)*e.dir;
        if (start_to_perigee_dirarg > 0) {
            e.next_perigee_time = start_timestep + start_to_perigee_dirarg/(2*M_PI)*e.period;
            e.next_apogee_time = e.next_perigee_time + e.period/2;
        }
        else {
            start_to_perigee_dirarg += 2*M_PI;
            e.next_perigee_time = start_timestep + start_to_perigee_dirarg/(2*M_PI)*e.period;
            e.next_apogee_time = e.next_perigee_time - e.period/2;
        }
    }

    printf("guess_ellipse: ");
    if (target_id == FATE_ID)
        printf("fate");
    else if (target_id == STATION_ID)
        printf("station");
    else
        printf("target %d", target_id);

    printf(": another_focus=(%.0f, %.0f), major_radius=%.0f, perigee=(%.0f, %.0f), period=%.3f\n",
           e.another_focus.real(), e.another_focus.imag(), e.major_radius,
           e.perigee.real(), e.perigee.imag(), e.period);

    return e;
}


/*/


ellipse_t guess_ellipse_by_earth(OVM& ovm_original, int target_id) {
    ellipse_t e;
    OVM ovm(ovm_original);
    e.next_apogee_time = -1;
    e.next_perigee_time = -1;
    e.apogee = e.perigee = P();
    for(int i = 0; i < 3000000; ++i){
        int now = ovm.current_timestep();
        P cur = CURRENT_POSITION();
        P prev = PREVIOUS_POSITION();
        ovm.run();
        P next = CURRENT_POSITION();
        if (norm(prev) < norm(cur) && norm(next) < norm(cur)) {
          cerr << "DEBUG: apogee " << now << " : " << cur << endl;
            if (e.next_apogee_time != -1) {
                e.period = now - e.next_apogee_time;
                break;
            }
            e.apogee = cur;
            e.next_apogee_time = now;
        }
        if(norm(cur) < norm(prev) && norm(cur) < norm(next)) {
            //cerr << "DEBUG: perigee " << now << " : " << cur << endl;
            if (e.next_perigee_time == -1) {
                e.perigee = cur;
                e.next_perigee_time = now;
            }
        }
    }
    e.major_radius = abs(e.apogee - e.perigee) * 0.5;
    e.another_focus = e.apogee + e.perigee;

    printf("guess_ellipse: ");
    if (target_id == FATE_ID)
        printf("fate");
    else if (target_id == STATION_ID)
        printf("station");
    else
        printf("target %d", target_id);

    if (e.perigee == P())
        printf(" [FAILED!] ");

    printf(": another_focus=(%.0f, %.0f), major_radius=%.0f, perigee=(%.0f, %.0f), period=%.3f\n",
           e.another_focus.real(), e.another_focus.imag(), e.major_radius,
           e.perigee.real(), e.perigee.imag(), e.period);

    return e;
}


//*/

