#include <iostream>
#include <cstdlib>
#include "ovm.hpp"
#include "transitions.hpp"
#include "autofollow.hpp"
#include "guess_ellipse.hpp"
#include "simulate.hpp"

using namespace std;


const P I(0, 1);

inline double delta_orbit(P cur, P perigee) {
    return abs(abs(perigee)-abs(cur)) / abs(perigee);
}

void meet_greet(OVM& ovm, int target_id) {

    // 最初から重なっていたらとくにすることはない
    if (ovm.target_collected(target_id))
        return;
    wait_perigee(ovm);
    if (ovm.target_collected(target_id))
        return;

    enter_circular_orbit(ovm);

    ellipse_t e = guess_ellipse_by_earth(ovm, target_id);
    //cerr << "DEBUG: " << e.perigee << " " << ovm.current_target_position(target_id) << endl;

    if (delta_orbit(ovm.current_fate_position(), e.perigee) < 0.0001) {
        // 軌道が近すぎるので、いったん外に出る
        double fire_fuel = 100;
        printf("meet_greet: too simlar orbit, going to outer orbit now using fuel %.0f\n", fire_fuel);
        ovm.run(fire_fuel * (ovm.current_fate_velocity()/abs(ovm.current_fate_velocity())));
        while(delta_orbit(ovm.current_fate_position(), e.perigee) < 0.0001)
            ovm.run();
        enter_circular_orbit(ovm);
    }

    double meet_time;
    P meet_point;
    for(meet_time = e.next_perigee_time; ; meet_time += e.period) {
        meet_point = simulate_with_moon(ovm.current_target_position(target_id), ovm.current_target_velocity(target_id),
                                        ovm.current_moon_position(), ovm.current_moon_velocity(),
                                        (int)(meet_time - ovm.current_timestep()));
        if (transit_circular_orbit(ovm, meet_point, meet_time))
            break;
    }
    while (meet_time - ovm.current_timestep() > 0.5) {
        ovm.run();
    }
    //cerr << "DEBUG2: " << meet_point << " " << ovm.current_target_position(target_id) << endl;
    printf("meet_greet: ");
    ovm.print_status(target_id);

    auto_follow(ovm, target_id, 12);
}


int main(int argc, char** argv) {

    if (argc == 1) {
        printf("usage: %s <scenario ID>\n", argv[0]);
        return 0;
    }

    OVM ovm(atoi(argv[1]));
    ovm.set_auto_save(true);
    //ovm.set_auto_vis(100);
    ovm.run();
    ovm.run();

    int targets = 7;
    for(int target_id = 0; !ovm.finished() && target_id < targets; target_id++) {
        printf("***** %7d: target: %d\n", ovm.current_timestep(), target_id);
        ovm.print_status();
        meet_greet(ovm, target_id);
        if (target_id == targets-1)
            break;
        printf("***** %7d: returning to base\n", ovm.current_timestep());
        ovm.print_status();
        meet_greet(ovm, STATION_ID);
        enter_circular_orbit(ovm); // 地球に近いので、態勢をたてなおす
    }

    printf("waiting to finish...\n");
    while(!ovm.finished()) {
        if (ovm.current_timestep() % 1000 == 0)
            printf("%d\r", ovm.current_timestep());
        ovm.run();
    }
    printf("last status: ");
    ovm.print_status();
    cout << "moon: " << ovm.current_moon_position() << endl;

    cout << "score: " << ovm.current_score() << endl;

    return 0;
}
