#include <algorithm>
#include <complex>
#include <iomanip>
#include <iostream>
#include <cassert>
#include <cmath>
#include <cstring>
#include "ovm.hpp"

using namespace std;

#include "simulate.hpp"


P simulate(P current_fate_position, P current_fate_velocity, int steps) {
    while(steps-- > 0) {
        P current_gravity = -( current_fate_position/abs(current_fate_position)*(G*Me/norm(current_fate_position)) );
        P next_fate_position = current_fate_position + current_fate_velocity + current_gravity / 2.0;
        P next_gravity = -( next_fate_position/abs(next_fate_position)*(G*Me/norm(next_fate_position)) );
        P next_fate_velocity = current_fate_velocity + (current_gravity + next_gravity) / 2.0;
        current_fate_position = next_fate_position;
        current_fate_velocity = next_fate_velocity;
    }
    return current_fate_position;
}

void simulate_v(P current_fate_position, P current_fate_velocity,
		int steps, P& next_position, P& next_v) {
    while(steps-- > 0) {
        P current_gravity = -( current_fate_position/abs(current_fate_position)*(G*Me/norm(current_fate_position)) );
        P next_fate_position = current_fate_position + current_fate_velocity + current_gravity / 2.0;
        P next_gravity = -( next_fate_position/abs(next_fate_position)*(G*Me/norm(next_fate_position)) );
        P next_fate_velocity = current_fate_velocity + (current_gravity + next_gravity) / 2.0;
        current_fate_position = next_fate_position;
        current_fate_velocity = next_fate_velocity;
    }
    next_position = current_fate_position;
    next_v = current_fate_velocity;
}

P simulate_with_moon(P current_fate_position, P current_fate_velocity, P current_moon_position, P current_moon_velocity, int steps) {
    while(steps-- > 0) {
        P current_fate_moon_position = current_fate_position - current_moon_position;
        P current_gravity = -( current_fate_position/abs(current_fate_position)*(G*Me/norm(current_fate_position)) +
                               current_fate_moon_position/abs(current_fate_moon_position)*(G*Mm/norm(current_fate_moon_position)) );
        P next_fate_position = current_fate_position + current_fate_velocity + current_gravity / 2.0;
        P current_moon_gravity = -( current_moon_position/abs(current_moon_position)*(G*Me/norm(current_moon_position)) );
        P next_moon_position = current_moon_position + current_moon_velocity + current_moon_gravity / 2.0;
        P next_fate_moon_position = next_fate_position - next_moon_position;
        P next_gravity = -( next_fate_position/abs(next_fate_position)*(G*Me/norm(next_fate_position)) +
                               next_fate_moon_position/abs(next_fate_moon_position)*(G*Mm/norm(next_fate_moon_position)) );
        P next_fate_velocity = current_fate_velocity + (current_gravity + next_gravity) / 2.0;
        P next_moon_gravity = -( next_moon_position/abs(next_moon_position)*(G*Me/norm(next_moon_position)) );
        P next_moon_velocity = current_moon_velocity + (current_moon_gravity + next_moon_gravity) / 2.0;
        current_fate_position = next_fate_position;
        current_fate_velocity = next_fate_velocity;
        current_moon_position = next_moon_position;
        current_moon_velocity = next_moon_velocity;
    }
    return current_fate_position;
}
