#include <cstdio>
#include <cassert>
#include <cstring>

using namespace std;

#define TEAM_ID 652
#define CAFEBABE 0xCAFEBABE

void vm_run_1(int scenario_id, double* mem, double* input, double* output, int& status);
void vm_run_2(int scenario_id, double* mem, double* input, double* output, int& status);
void vm_run_3(int scenario_id, double* mem, double* input, double* output, int& status);
void vm_run_4(int scenario_id, double* mem, double* input, double* output, int& status);
extern double vm_init_memmap_1[];
extern double vm_init_memmap_2[];
extern double vm_init_memmap_3[];
extern double vm_init_memmap_4[];
extern int vm_memsize_1;
extern int vm_memsize_2;
extern int vm_memsize_3;
extern int vm_memsize_4;

double mem[0x4000];
double input[0x4000];
double output[0x4000];

unsigned int read_uint() {
    unsigned int x;
    fread(&x, sizeof(x), 1, stdin);
    return x;
}
double read_double() {
    double x;
    fread(&x, sizeof(x), 1, stdin);
    return x;
}

int main() {
    unsigned int cafebabe = read_uint();
    unsigned int team_id = read_uint();
    unsigned int scenario_id = read_uint();

    printf("cafebabe: 0x%08x\n", cafebabe);
    printf("team ID: %d\n", team_id);
    printf("scenario ID: %d\n", scenario_id);

    assert(cafebabe == CAFEBABE);
    assert(team_id == TEAM_ID);

    switch(scenario_id/1000) {
    case 1: memcpy(mem, vm_init_memmap_1, sizeof(double)*vm_memsize_1); break;
    case 2: memcpy(mem, vm_init_memmap_2, sizeof(double)*vm_memsize_2); break;
    case 3: memcpy(mem, vm_init_memmap_3, sizeof(double)*vm_memsize_3); break;
    case 4: memcpy(mem, vm_init_memmap_4, sizeof(double)*vm_memsize_4); break;
    }

    int status = 0;
    int last_timestep = 0;
    for(;;) {
        int timestep = read_uint();
        while(last_timestep < timestep) {
            switch(scenario_id/1000) {
            case 1: vm_run_1(scenario_id, mem, input, output, status); break;
            case 2: vm_run_2(scenario_id, mem, input, output, status); break;
            case 3: vm_run_3(scenario_id, mem, input, output, status); break;
            case 4: vm_run_4(scenario_id, mem, input, output, status); break;
            }
            last_timestep++;
        }
        int n = read_uint();
        if (n == 0)
            break;
        for(int i = 0; i < n; i++) {
            unsigned int addr = read_uint();
            double value = read_double();
            input[addr] = value;
        }
        assert(output[0] == 0);
    }
    assert(fgetc(stdin) == EOF);
    assert(output[0] != 0);
    printf("score = %.5f\n", output[0]);

    return 0;
}
