package jp.dip.icfp2009;

import java.awt.event.MouseEvent;

import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;

public class VisualFateMouseAdapter extends MouseInputAdapter {

	private final VisualFateDocument document;

	public VisualFateMouseAdapter(VisualFateDocument document) {
		if (document == null) {
			throw new NullPointerException("Document is null.");
		}

		this.document = document;
		this.prevX = 0;
		this.prevY = 0;
	}
	
	private int prevX;
	private int prevY;
	
	@Override
	public void mousePressed(MouseEvent e) {
		prevX = e.getX();
		prevY = e.getY();
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();

		int dx = x - prevX;
		int dy = y - prevY;
		
		double scale = document.getScale();
		double centerX = document.getCenterX() - scale * dx; 
		double centerY = document.getCenterY() + scale * dy;
		document.setCenter(centerX, centerY);
		
		prevX = x;
		prevY = y;
		SwingUtilities.getRoot(e.getComponent()).repaint();
	}
}
