package jp.dip.icfp2009;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.Map;

import javax.swing.JPanel;

@SuppressWarnings("serial")
public class VisualFatePanel extends JPanel {

	private final VisualFateDocument document;
	
	public VisualFatePanel(VisualFateDocument document) {
		if (document == null) {
			throw new NullPointerException("document is null.");
		}

		this.document = document;
		super.setDoubleBuffered(true);
		super.setBackground(Color.WHITE);
		super.setPreferredSize(new Dimension(10000, 10000));
		VisualFateMouseAdapter mouseAdapter = new VisualFateMouseAdapter(document);
		super.addMouseListener(mouseAdapter);
		super.addMouseMotionListener(mouseAdapter);
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		Color colorBackup = g.getColor();
		try {
			int width = super.getWidth();
			int height = super.getHeight();

			int centerX = width / 2;
			int centerY = height / 2;
			double scale = document.getScale();
			
			double docCenterX = document.getCenterX();
			double docCenterY = document.getCenterY();
			
			// Draw the earth.
			{
				g.setColor(Color.BLUE);
				int earth_radius = (int)(VisualFateDocument.EARTH_RADIUS / scale);
				int earthCenterX = (int)(-docCenterX / scale) + centerX;
				int earthCenterY = (int)(docCenterY / scale) + centerY;
				
				g.fillOval(earthCenterX - earth_radius, earthCenterY - earth_radius,
						   earth_radius * 2, earth_radius * 2);
			}

			// Draw entities.
			{
				int currentTime = document.getCurrentTime();
				Map<String, VisualFateDocument.Entity> entityMap = document.getEntityMap();
				for (VisualFateDocument.Entity entity : entityMap.values()) {
					// Draw the trajectory.
					{
						// Mark INT_MIN as un-initialized.
						int prevPanelX = Integer.MIN_VALUE;
						int prevPanelY = Integer.MIN_VALUE;
						for (VisualFateDocument.State state : entity.getStateList()) {
							double x = state.getX() - docCenterX;
							double y = state.getY() - docCenterY;
							int panelX = (int)(x / scale) + centerX;
							int panelY = (int)(-y / scale) + centerY;
							if (prevPanelX != Integer.MIN_VALUE && prevPanelY != Integer.MIN_VALUE) {
								g.setColor(Color.GRAY);
								g.drawLine(prevPanelX, prevPanelY, panelX, panelY);
							}
							double vx = state.getVx();
							double vy = state.getVy();
							if (vx != 0 || vy != 0) {
								int panelVx = (int)(vx / scale * 10000);
								int panelVy = (int)(-vy / scale * 10000);
								g.setColor(Color.ORANGE);
								g.drawLine(panelX, panelY, panelX - panelVx, panelY - panelVy);
							}
							
							prevPanelX = panelX;
							prevPanelY = panelY;
						}
					}
					
					// Draw the current point.
					{
						VisualFateDocument.State state = entity.getStateAt(currentTime);
						double x = state.getX() - docCenterX;
						double y = state.getY() - docCenterY;
						int panelX = (int)(x / scale) + centerX;
						int panelY = (int)(-y / scale) + centerY;
						g.setColor(Color.RED);
						g.fillOval(panelX - 3, panelY - 3, 6, 6);
						g.setColor(Color.BLACK);
						g.drawString(entity.getName(), panelX + 5, panelY + 12);
					}
				}
			}
		} finally {
			g.setColor(colorBackup);
		}
	}
}
