package jp.dip.icfp2009;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicSliderUI;

@SuppressWarnings("serial")
public class VisualFateSliderPanel extends JPanel {

	private static class TimeSlider extends JSlider {
		public TimeSlider(VisualFateDocument document, JLabel valueLabel) {
			super.setMaximum(0);
			super.setMinimum(0);
			super.setValue(0);
			super.setUI(new BasicSliderUI(this) {
				public void scrollByUnit(int direction) {
			        synchronized(slider)    {
			            int oldValue = slider.getValue();
			            int delta = 100 * ((direction > 0) ? POSITIVE_SCROLL : NEGATIVE_SCROLL);
			            slider.setValue(oldValue + delta);  
			        }       
				}
			});
			super.addChangeListener(new TimeChangeListener(this, document, valueLabel));
		}

		private static class TimeChangeListener implements ChangeListener {
			private TimeSlider slider;
			private VisualFateDocument document;
			private JLabel valueLabel;
			
			public TimeChangeListener(TimeSlider slider, VisualFateDocument document, JLabel valueLabel) {
				if (slider == null) {
					throw new NullPointerException("Slider is null.");
				}
				if (document == null) {
					throw new NullPointerException("document is null.");
				}
				if (valueLabel == null) {
					throw new NullPointerException("valueLabel is null.");
				}
				this.slider = slider;
				this.document = document;
				this.valueLabel = valueLabel;
			}

			@Override
			public void stateChanged(ChangeEvent e) {
				int value = slider.getValue();
				document.setCurrentTime(value);
				valueLabel.setText("[" + value + "]");
				SwingUtilities.getRoot(slider).repaint();
			}
		}
	
	}

	public static class TimePanel extends JPanel {
		private final TimeSlider timeSlider;
		
		private TimePanel(VisualFateDocument document) {
			JLabel label = new JLabel("Time:");
			JLabel valueLabel = new JLabel("[0]");
			this.timeSlider = new TimeSlider(document, valueLabel);
			super.add(label);
			super.add(this.timeSlider);
			super.add(valueLabel);
		}

		public TimeSlider getTimeSlider() {
			return timeSlider;
		}
	}
	
	
	private static class ScaleSlider extends JSlider {
		public ScaleSlider(VisualFateDocument document) {
			super.setMaximum(300);
			super.setMinimum(1);
			super.setValue(30);
			super.addChangeListener(new ScaleChangeListener(this, document));
		}

		private static class ScaleChangeListener implements ChangeListener {
			private ScaleSlider slider;
			private VisualFateDocument document;
			private ScaleChangeListener(ScaleSlider slider, VisualFateDocument document) {
				if (slider == null) {
					throw new NullPointerException("Slider is null.");
				}
				if (document == null) {
					throw new NullPointerException("document is null.");
				}
				this.slider = slider;
				this.document = document;
			}

			@Override
			public void stateChanged(ChangeEvent e) {
				double value = slider.getValue() / 30.0;
				value = 10 * value * value;
				document.setScale(VisualFateDocument.EARTH_RADIUS / 1000 * value * 2);
				SwingUtilities.getRoot(slider).repaint();
			}
		}
	}

	public static class CenterResetButton extends JButton {
		private CenterResetButton(VisualFateFrame frame, VisualFateDocument document) {
			super.setText("Reset the Center");
			this.addActionListener(new CenterResetAction(frame, document));
		}

		private static class CenterResetAction implements ActionListener {
			private final VisualFateFrame frame;
			private final VisualFateDocument document;
			
			private CenterResetAction(VisualFateFrame frame, VisualFateDocument document) {
				if (frame == null) {
					throw new NullPointerException("Frame is null.");
				}
				if (document == null) {
					throw new NullPointerException("Document is null.");
				}
				this.frame = frame;
				this.document = document;
			}
			
			@Override
			public void actionPerformed(ActionEvent e) {
				document.setCenter(0, 0);
				frame.repaint();
			}
		}
	}
	
	public static class ScalePanel extends JPanel {
		private final ScaleSlider scaleSlider;
		public ScalePanel(VisualFateFrame frame, VisualFateDocument document) {
			super.add(new JLabel("Scale:"));
			this.scaleSlider = new ScaleSlider(document);
			super.add(this.scaleSlider);
			super.add(new CenterResetButton(frame, document));
		}
		public ScaleSlider getScaleSlider() {
			return scaleSlider;
		}
	}
	
	private final ScalePanel scalePanel;
	private final TimePanel timePanel;
	
	public VisualFateSliderPanel(VisualFateFrame frame, VisualFateDocument document) {
		this.scalePanel = new ScalePanel(frame, document);
		this.timePanel = new TimePanel(document);
		this.add(this.scalePanel);
		this.add(this.timePanel);
	}

	public ScalePanel getScalePanel() {
		return scalePanel;
	}
	
	public TimePanel getTimePanel() {
		return timePanel;
	}
}
